import java.awt.image.BufferedImage;
import java.awt.image.DataBufferInt;

import processing.core.PImage;

import com.amd.aparapi.Kernel;
import com.amd.aparapi.Range;

/**
 * LifeKernel represents the data parallel algorithm describing by Conway's game
 * of life.
 * 
 * http://en.wikipedia.org/wiki/Conway's_Game_of_Life
 * 
 * We examine the state of each pixel and its 8 neighbors and apply the
 * following rules.
 * 
 * if pixel is dead (off) and number of neighbors == 3 { pixel is turned on }
 * else if pixel is alive (on) and number of neighbors is neither 2 or 3 pixel
 * is turned off }
 * 
 * We use an image buffer which is 2*width*height the size of screen and we use
 * fromBase and toBase to track which half of the buffer is being mutated for
 * each pass. We basically copy from getGlobalId()+fromBase to
 * getGlobalId()+toBase;
 * 
 * 
 * Prior to each pass the values of fromBase and toBase are swapped.
 * 
 */

public class LifeKernelP5 extends Kernel {

	private static final int ALIVE = -1;

	private static final int DEAD = -16777216;

	private final int[] imageData;

	private final int width;

	private final int height;

	private final Range range;

	public LifeKernelP5(int _width, int _height, int[] pixels) {
		width = _width;
		height = _height;
		imageData = pixels;
		range = Range.create(width * height, 256);// , 256);
		System.out.println("range = " + range);
	}

	@Override
	public void run() {
		int gid = getGlobalId();
		int to = gid;// + toBase;
		int from = gid;// + fromBase;
		int x = gid % width;
		int y = gid / width;

		if ((x == 0 || x == width - 1 || y == 0 || y == height - 1)) {
			// This pixel is on the border of the view, just keep existing value
			imageData[to] = imageData[from];
		} else {
			// Count the number of neighbors. We use (value&1x) to turn pixel
			// value into either 0 or 1

			int neighbors = 0;
			if (imageData[from - 1] == ALIVE)
				neighbors++; // EAST
			if (imageData[from + 1] == ALIVE)
				neighbors++; // EAST
			if (imageData[from - width - 1] == ALIVE)
				neighbors++; // EAST
			if (imageData[from - width] == ALIVE)
				neighbors++; // EAST
			if (imageData[from - width + 1] == ALIVE)
				neighbors++; // EAST
			if (imageData[from + width - 1] == ALIVE)
				neighbors++; // EAST
			if (imageData[from + width] == ALIVE)
				neighbors++; // EAST
			if (imageData[from + width + 1] == ALIVE)
				neighbors++; // EAST

			// The game of life logic
			if (neighbors == 3 || (neighbors == 2 && imageData[from] == ALIVE)) {
				imageData[gid] = ALIVE;
			} else {
				imageData[gid] = DEAD;
			}

		}

	}

	public void nextGeneration() {
		// swap fromBase and toBase
		execute(range);
	}

	public int[] getImageData() {
		return imageData;
	}
}
